@@ -1,28 +1,17 @@
-from .data import (
-    AggregateExchangeRatePrevote,
-    AggregateExchangeRateVote,
-    ExchangeRatePrevote,
-    ExchangeRateVote,
-)
+from .data import AggregateExchangeRatePrevote, AggregateExchangeRateVote
 from .msgs import (
     MsgAggregateExchangeRatePrevote,
     MsgAggregateExchangeRateVote,
     MsgDelegateFeedConsent,
-    MsgExchangeRatePrevote,
-    MsgExchangeRateVote,
     aggregate_vote_hash,
     vote_hash,
 )
 
 __all__ = [
-    "ExchangeRatePrevote",
-    "ExchangeRateVote",
     "AggregateExchangeRatePrevote",
     "AggregateExchangeRateVote",
     "vote_hash",
     "aggregate_vote_hash",
-    "MsgExchangeRatePrevote",
-    "MsgExchangeRateVote",
     "MsgDelegateFeedConsent",
     "MsgAggregateExchangeRatePrevote",
     "MsgAggregateExchangeRateVote",
