@@ -1,23 +1,21 @@
+from terra_sdk.core.authz import (
+    MsgExecAuthorized,
+    MsgGrantAuthorization,
+    MsgRevokeAuthorization,
+)
 from terra_sdk.core.bank import MsgMultiSend, MsgSend
 from terra_sdk.core.distribution import (
     MsgFundCommunityPool,
-    MsgModifyWithdrawAddress,
+    MsgSetWithdrawAddress,
     MsgWithdrawDelegationReward,
     MsgWithdrawValidatorCommission,
 )
 from terra_sdk.core.gov.msgs import MsgDeposit, MsgSubmitProposal, MsgVote
 from terra_sdk.core.market import MsgSwap, MsgSwapSend
-from terra_sdk.core.msgauth import (
-    MsgExecAuthorized,
-    MsgGrantAuthorization,
-    MsgRevokeAuthorization,
-)
 from terra_sdk.core.oracle import (
     MsgAggregateExchangeRatePrevote,
     MsgAggregateExchangeRateVote,
     MsgDelegateFeedConsent,
-    MsgExchangeRatePrevote,
-    MsgExchangeRateVote,
 )
 from terra_sdk.core.slashing import MsgUnjail
 from terra_sdk.core.staking import (
@@ -28,25 +26,49 @@ from terra_sdk.core.staking import (
     MsgUndelegate,
 )
 from terra_sdk.core.wasm import (
+    MsgClearContractAdmin,
     MsgExecuteContract,
     MsgInstantiateContract,
+    MsgMigrateCode,
     MsgMigrateContract,
     MsgStoreCode,
-    MsgUpdateContractOwner,
+    MsgUpdateContractAdmin,
+)
+from terra_sdk.core.ibc.msgs import (
+    MsgCreateClient,
+    MsgUpdateClient,
+    MsgUpgradeClient,
+    MsgSubmitMisbehaviour,
+    MsgConnectionOpenInit,
+    MsgConnectionOpenTry,
+    MsgConnectionOpenAck,
+    MsgConnectionOpenConfirm,
+    MsgChannelOpenInit,
+    MsgChannelOpenTry,
+    MsgChannelOpenAck,
+    MsgChannelOpenConfirm,
+    MsgChannelCloseInit,
+    MsgChannelCloseConfirm,
+    MsgRecvPacket,
+    MsgTimeout,
+    MsgAcknowledgement
+)
+from terra_sdk.core.ibc_transfer import (
+    MsgTransfer
 )
 
-from .base import create_demux
+from .base import create_demux, create_demux_proto
 
 bank_msgs = [MsgSend, MsgMultiSend]
 distribution_msgs = [
     MsgFundCommunityPool,
-    MsgModifyWithdrawAddress,
+    MsgSetWithdrawAddress,
     MsgWithdrawDelegationReward,
     MsgWithdrawValidatorCommission,
 ]
 gov_msgs = [MsgDeposit, MsgSubmitProposal, MsgVote]
 market_msgs = [MsgSwap, MsgSwapSend]
-msgauth_msgs = [
+authz_msgs = [
     MsgExecAuthorized,
     MsgGrantAuthorization,
     MsgRevokeAuthorization,
@@ -55,8 +77,6 @@ oracle_msgs = [
     MsgAggregateExchangeRatePrevote,
     MsgAggregateExchangeRateVote,
     MsgDelegateFeedConsent,
-    MsgExchangeRatePrevote,
-    MsgExchangeRateVote,
 ]
 slashing_msgs = [MsgUnjail]
 staking_msgs = [
@@ -67,11 +87,36 @@ staking_msgs = [
     MsgUndelegate,
 ]
 wasm_msgs = [
-    MsgExecuteContract,
-    MsgInstantiateContract,
-    MsgMigrateContract,
     MsgStoreCode,
-    MsgUpdateContractOwner,
+    MsgMigrateCode,
+    MsgInstantiateContract,
+    MsgExecuteContract,
+    MsgMigrateContract,
+    MsgUpdateContractAdmin,
+    MsgClearContractAdmin,
+]
+
+ibc_transfer_msgs = [
+    MsgTransfer
+]
+ibc_msgs = [
+    MsgCreateClient,
+    MsgUpdateClient,
+    MsgUpgradeClient,
+    MsgSubmitMisbehaviour,
+    MsgConnectionOpenInit,
+    MsgConnectionOpenTry,
+    MsgConnectionOpenAck,
+    MsgConnectionOpenConfirm,
+    MsgChannelOpenInit,
+    MsgChannelOpenTry,
+    MsgChannelOpenAck,
+    MsgChannelOpenConfirm,
+    MsgChannelCloseInit,
+    MsgChannelCloseConfirm,
+    MsgRecvPacket,
+    MsgTimeout,
+    MsgAcknowledgement
 ]
 
 parse_msg = create_demux(
@@ -84,5 +129,21 @@ parse_msg = create_demux(
         *slashing_msgs,
         *staking_msgs,
         *wasm_msgs,
+        *ibc_msgs,
+        *ibc_transfer_msgs
     ]
 )
+parse_proto = create_demux_proto(
+    [
+        *bank_msgs,
+        *distribution_msgs,
+        *gov_msgs,
+        *market_msgs,
+        *oracle_msgs,
+        *slashing_msgs,
+        *staking_msgs,
+        *wasm_msgs,
+        *ibc_msgs,
+        *ibc_transfer_msgs
+    ]
+)
\ No newline at end of file
